# test_vocabulary_generator.py

from recluse import vocabulary_generator
import unittest, StringIO, subprocess, bz2


class VocabularyGeneratorTest(unittest.TestCase):

    @classmethod
    def setUpClass(self):
        self.vg = vocabulary_generator.VocabularyGenerator()
        self.text_file_name = 'recluse/test/data/small_westbury.txt.bz2'
        self.size = 20

    def test_no_merge(self):
        out_file_obj = StringIO.StringIO()
        self.vg.generate_vocabulary(self.text_file_name, self.size, out_file_obj)
        words = out_file_obj.getvalue()
        self.assertEqual(words, u'the\n,\n<s>\n</s>\n.\nof\nand\nin\n"\nto\na\nis\n-\nas\nthat\n<4-digit-integer>\nfor\nwas\nwith\n)\n'), words

    def test_merge(self):
        out_file_obj = StringIO.StringIO()
        self.vg.lines_per_chunk = 500
        self.vg.generate_vocabulary(self.text_file_name, self.size, out_file_obj)
        words = out_file_obj.getvalue()
        self.assertEqual(words, u'the\n,\n<s>\n</s>\n.\nof\nand\nin\n"\nto\na\nis\n-\nas\nthat\n<4-digit-integer>\nfor\nwas\nwith\n)\n'), words

    def test_command_line(self):
        generate_vocab = subprocess.Popen(['python', 'recluse/vocabulary_generator.py', self.text_file_name, str(self.size)], stdin=-1, stdout=-1, stderr=-1)
        test_data_reader = bz2.BZ2File('recluse/test/data/small_westbury.txt.bz2', 'r')
        (stdoutdata, stderrdata) = generate_vocab.communicate()
        self.assertEqual(generate_vocab.returncode, 0)
        self.assertEqual(stdoutdata, 'the\n,\n<s>\n</s>\n.\nof\nand\nin\n"\nto\na\nis\n-\nas\nthat\n<4-digit-integer>\nfor\nwas\nwith\n)\n'), stdoutdata
        


if __name__ == '__main__':
    unittest.main()



# L. Amber Wilcox-O'Hearn 2011
# test_vocabulary_cutter.py

import unittest, StringIO, subprocess
from recluse import vocabulary_cutter


class VocabularyCutterTest(unittest.TestCase):

    def test_cut_vocabulary(self):

        infile_obj = open('recluse/test/data/unigram_counts', 'r')
        outfile_obj = StringIO.StringIO()

        vc = vocabulary_cutter.VocabularyCutter(infile_obj, outfile_obj)
        vc.cut_vocabulary(5)

        assert outfile_obj.getvalue() == "being\nheld\nGer\ndeficits\nChris\n", outfile_obj.getvalue()

    def test_commandline(self):

        cut_vocab = subprocess.Popen(['python', 'recluse/vocabulary_cutter.py', '5'], stdin=-1, stdout=-1, stderr=-1)
        infile_obj = open('recluse/test/data/unigram_counts', 'r')
        (stdoutdata, stderrdata) = cut_vocab.communicate(input=infile_obj.read())
        self.assertEqual(cut_vocab.returncode, 0)
        self.assertEqual(stdoutdata, "being\nheld\nGer\ndeficits\nChris\n")


if __name__ == '__main__':
    unittest.main()



# 2012 L. Amber Wilcox-O'Hearn
# NLTKBasedSegmenterTokeniser.py

import nltk
import re, sys, codecs, unicodedata, string

class NLTKBasedSegmenterTokeniser():

    """
    This Segementer/Tokeniser is customised in the following ways:

    1. It has parameters optimised for Wikipedia text (i.e. text that
    is inconsistent in its spelling of abbreviations, has an immense
    vocabulary, and is stylistically diverse and non-standard).

    2. This version replaces strings of digits with a special string
    that has only a digit count. TODO: make this optional.

    3. It has a mode that emits lists of boundary positions and token
    replacements rather than putting out transformed text, in order to
    preserve the original.  EXCEPTION: note that in this version
    non-linebreaking whitespace is always collapsed to a single space,
    however, which violates complete reversibility.

    4. This version always starts by training on the supplied text.
    New text can also be supplied after the training (though this is
    not yet tested), but the training is not currenty loadable or
    savable.

    """

    def __init__(self, infile_obj):
        self.unicode_infile_obj = codecs.getreader('utf-8')(infile_obj)
        self.text = self.unicode_infile_obj.read()
        assert isinstance(self.text, unicode)
        assert len(self.text) > 0
        trainer = nltk.tokenize.punkt.PunktTrainer()
        # The following are optimisations recommended by NLTK
        # developers for Wikipedia:
        trainer.ABBREV = .15
        trainer.IGNORE_ABBREV_PENALTY = True
        trainer.INCLUDE_ALL_COLLOCS = True
        trainer.MIN_COLLOC_FREQ = 10
        # ---------------------------------------------------
        trainer.train(self.text)
        self.sbd = nltk.tokenize.punkt.PunktSentenceTokenizer(trainer.get_params())

    def is_an_initial(self, word):
        if len(word) == 2 and unicodedata.category(word[0])[0] == 'L' and word[1] == u'.':
            return True
        return False
    def multi_char_word_and_starts_with_a_capital(self, word):
        if len(word) > 1 and unicodedata.category(word[0]) == 'Lu':
            return True
        return False

    def apply_ugly_hack_to_reattach_wrong_splits_in_certain_cases_with_initials(self, lines):
        """
        NLTK currently splits sentences between 2 initials.  Hacking
        those back together.  Also has the effect of collapsing
        whitespace to a single space char.
        """
        lines = list(lines)
        if len(lines) == 0: return []
        reattached_lines = []
        i = 0
        current_line = lines[i].split()
        while i < len(lines) - 1:
            reattach = False
            next_line = lines[i+1].split()
            last_word = current_line[-1]
            next_line_starts_with_a_capital = False
            first_word_of_next_line = next_line[0]
            if len(first_word_of_next_line) > 1 and unicodedata.category(first_word_of_next_line[0]) == 'Lu':
                next_line_starts_with_a_capital = True
            if self.is_an_initial(last_word):
                nltk_ortho_context = self.sbd._params.ortho_context[first_word_of_next_line.lower()]
                if unicodedata.category(first_word_of_next_line[0])[0] != 'L':
                    reattach = True
                # The following is an ugly and imperfect hack.  See mailing list for nltk.
                elif self.multi_char_word_and_starts_with_a_capital(first_word_of_next_line) and \
                        nltk_ortho_context <= 46 or \
                        self.is_an_initial(first_word_of_next_line):
                    reattach = True

            if reattach:
                    current_line += next_line
            else:
                reattached_lines.append(u' '.join(current_line))
                current_line = next_line
            i += 1 
        reattached_lines.append(u' '.join(current_line))
        return reattached_lines
                
    def lists_of_internal_token_boundaries_and_special_tokens(self, line):
        """
        If any chars are unicode punctuation and not periods, put
        boundary marks around them except at the beginning and end of
        the line, and unless it is a contraction or an inter-numeric
        comma.  Also put boundaries around ellipses.  Strings of
        digits are listed in the substitutions list with
        '<n-digit-integer>' I've traded some readability for doing it
        in one pass.
        """

        boundaries_set = set([])
        special_tokens = []
        digit_length = 0
        number_punct = False
        if unicodedata.category(line[0]) == 'Nd':
            digit_length = 1
        elif unicodedata.category(line[0])[0] in 'PS' and line[0] != '.':
            boundaries_set.add(1)
        for i in [x+1 for x in range(len(line)-1)]:
            if unicodedata.category(line[i]) == 'Nd':
                # We're in a digit string.
                digit_length += 1
            else:
                if digit_length > 0:
                    # Either there is a period or comma in the digit string or
                    # this ends the digit string. 
                    if (line[i] == '.' or line[i] == ',') and i < len(line)-1 and unicodedata.category(line[i+1]) =='Nd':
                        number_punct = True
                    special_tokens.append( (i - digit_length, digit_length,  u'<' + unicode(str(digit_length)) + u'-digit-integer>') )
                    digit_length = 0
                if unicodedata.category(line[i])[0] in 'PSZ' and line[i] != '.' and not number_punct:
                    if (line[i] == '\'' or line[i] == u'\xb4') and unicodedata.category(line[i-1])[0] == 'L' \
                        and i < len(line)-1 and unicodedata.category(line[i+1])[0] == 'L':
                        pass
                    else:
                        boundaries_set.update([i, i+1])
                number_punct = False
        if digit_length != 0:
            special_tokens.append( (len(line) - digit_length, digit_length,  u'<' + unicode(str(digit_length)) + u'-digit-integer>') )
            
        # Mark a boundary before sentence-final period if not an abbrevation
        i = len(line) - 1
        while unicodedata.category(line[i])[0] not in 'LN' and i >= 0:
            i -= 1
        if i >= 0:
            if i != len(line) - 1 and line[i+1] == '.':
                period_index = i+1
                # See if preceding token is an abbreviation or initial.
                while unicodedata.category(line[i]) != 'Zs' and i >= 0:
                    i -= 1
                token_index = i+1
                abbreviations = self.sbd._params.abbrev_types
                if line[token_index:period_index].lower() not in abbreviations and \
                        not (token_index + 1 == period_index and unicodedata.category(line[token_index]) == 'Lu'):
                    boundaries_set.add(period_index)

        # Find ellipses
        ellipses_index = line.find(u'...')
        while ellipses_index != -1:
            boundaries_set.add(ellipses_index)
            ellipses_index = line.find(u'...', ellipses_index+1)

        boundaries_set.discard(0)
        boundaries_set.discard(len(line))
        boundaries = list(boundaries_set)
        boundaries.sort()
        return boundaries, special_tokens

    def tokenised_text(self, sentence_and_token_information):

        text, boundaries, substitutions = sentence_and_token_information
        current_index = 0
        result = u''
        if len(boundaries) == 0: next_boundary = None
        else: 
            next_boundary_index = 0
            next_boundary = boundaries[0]
        if len(substitutions) == 0: next_sub = None
        else: 
            next_sub_index = 0
            next_sub, sub_length, sub_text = substitutions[0]

        while next_boundary is not None or next_sub is not None:
            while next_boundary is not None and (next_sub is None or next_boundary < next_sub):
                result += text[current_index:next_boundary] + u' '
                current_index = next_boundary
                next_boundary_index += 1
                if len(boundaries) < next_boundary_index + 1:
                    next_boundary = None
                else:
                    next_boundary = boundaries[next_boundary_index]
                if next_boundary is None and next_sub is None:
                    result += text[current_index:]
               
            if next_boundary is not None and next_sub is not None and next_boundary == next_sub:
                result += text[current_index:next_boundary] + u' ' + sub_text
                current_index = next_sub + sub_length
                next_boundary_index += 1
                if len(boundaries) < next_boundary_index + 1:
                    next_boundary = None
                else:
                    next_boundary = boundaries[next_boundary_index]
                next_sub_index += 1
                if len(substitutions) < next_sub_index + 1:
                    next_sub = None
                else:
                    next_sub, sub_length, sub_text = substitutions[next_sub_index]
                
            while next_sub is not None and (next_boundary is None or next_sub < next_boundary):
                result += text[current_index:next_sub] + sub_text
                current_index = next_sub + sub_length
                next_sub_index += 1
                if len(substitutions) < next_sub_index + 1:
                    next_sub = None
                else:
                    next_sub, sub_length, sub_text = substitutions[next_sub_index]
        return result
           
                    

    def segmented_and_tokenised(self, text=None, output_file_obj=None):
        """
        This function creates a generator, to avoid storing in RAM.
        If text is not supplied, the training text itself is used.
        If output_file_obj is supplied, the text is put out to that
        file with spaces inserted at the boundaries, and digit strings
        replaced with special tokens.
        """
        assert text is None or isinstance(text, unicode), text
        if text == None: text = self.text
        for line in (t for t in text.split('\n')):
            sentences = self.sbd.sentences_from_text(line, realign_boundaries=True)
            sentences = self.apply_ugly_hack_to_reattach_wrong_splits_in_certain_cases_with_initials(sentences)
            for sentence in sentences:
                sentence_and_token_information = (sentence,) + self.lists_of_internal_token_boundaries_and_special_tokens(sentence)
                yield sentence_and_token_information
                if output_file_obj:
                    unicode_outfile_obj = codecs.getwriter('utf-8')(output_file_obj)
                    lowered_text = sentence_and_token_information[0].lower()
                    lowered_sentence_and_token_information = (lowered_text, sentence_and_token_information[1], sentence_and_token_information[2])
                    unicode_outfile_obj.write(u' '.join(self.tokenised_text(lowered_sentence_and_token_information).split()) + u'\n')
        


if __name__ == '__main__':

    st = NLTKBasedSegmenterTokeniser(sys.stdin)
    for sti in st.segmented_and_tokenised(output_file_obj=sys.stdout):
        pass



# test_nltk_based_segmenter_tokeniser.py

# These are essentially all regression tests.

from recluse import nltk_based_segmenter_tokeniser
import unittest, StringIO, subprocess


class SegmenterAndTokeniserTest(unittest.TestCase):
    
    @classmethod
    def setUpClass(self):
        training_text_file = open('recluse/test/data/segmenter_training', 'r')
        self.segmenter_tokeniser = nltk_based_segmenter_tokeniser.NLTKBasedSegmenterTokeniser(training_text_file)

    def run_assertions(self, text_to_segment_tokenise, expected_list_of_tuple_output, expected_text_output):
        
        training_text_file = open('recluse/test/data/segmenter_training', 'r')
        out_file_obj = StringIO.StringIO()
        tuple_generator = self.segmenter_tokeniser.segmented_and_tokenised(text_to_segment_tokenise, out_file_obj)
        list_output = [x for x in tuple_generator]

        try:
            assert len(list_output) == len(expected_list_of_tuple_output)
            for i in range(len(list_output)):
                assert list_output[i] == expected_list_of_tuple_output[i]
        except AssertionError, exp:
            for i in range(len(expected_list_of_tuple_output)):
                for j in range(len(expected_list_of_tuple_output[i])):
                    if expected_list_of_tuple_output[i][j] == list_output[i][j]:
                        print "Matched"
                        print expected_list_of_tuple_output[i][j]
                    else:
                        print "Did not match"
                        print "Expected ", expected_list_of_tuple_output[i][j], "\nbut got  ", list_output[i][j]
            raise exp

        assert isinstance(out_file_obj.getvalue(), str), (type(out_file_obj.getvalue()), repr(out_file_obj.getvalue()))
        try:
            assert out_file_obj.getvalue() == expected_text_output
        except AssertionError, exp:
            x = out_file_obj.getvalue()
            for i in range(len(x)):
                if i >= len(expected_text_output) or x[i] != expected_text_output[i]: break
            print '\nMatching prefix of output and expected output: ', repr(x[:i])
            print '\noutput differs starting here: ', repr(x[i:])
            print '\nexpected: ', repr(expected_text_output[i:])
            raise exp

    def test_unicode(self):

        text_to_segment_tokenise = 'The term "anarchism" derives from the Greek \xe1\xbc\x84\xce\xbd\xce\xb1\xcf\x81\xcf\x87\xce\xbf\xcf\x82, "anarchos", meaning "without rulers", from the prefix \xe1\xbc\x80\xce\xbd- ("an-", "without") + \xe1\xbc\x80\xcf\x81\xcf\x87\xce\xae ("arch\xc3\xaa", "sovereignty, realm, magistracy") + -\xce\xb9\xcf\x83\xce\xbc\xcf\x8c\xcf\x82 ("-ismos", from the suffix -\xce\xb9\xce\xb6\xce\xb5\xce\xb9\xce\xbd, "-izein" "-izing").\nHere\xc2\xa0are\xc2\xa0some\xc2\xa0\nNBSPs!'.decode('utf-8')
        expected_list_of_tuple_output = [\
           ('The term "anarchism" derives from the Greek \xe1\xbc\x84\xce\xbd\xce\xb1\xcf\x81\xcf\x87\xce\xbf\xcf\x82, "anarchos", meaning "without rulers", from the prefix \xe1\xbc\x80\xce\xbd- ("an-", "without") + \xe1\xbc\x80\xcf\x81\xcf\x87\xce\xae ("arch\xc3\xaa", "sovereignty, realm, magistracy") + -\xce\xb9\xcf\x83\xce\xbc\xcf\x8c\xcf\x82 ("-ismos", from the suffix -\xce\xb9\xce\xb6\xce\xb5\xce\xb9\xce\xbd, "-izein" "-izing").'.decode('utf-8'), \
           [3, 4, 8, 9, 10, 19, 20, 21, 28, 29, 33, 34, 37, 38, 43, 44, 51, 52, 53, 54, 62, 63, 64, 65, 72, 73, 74, 81, 82, 88, 89, 90, 91, 95, 96, 99, 100, 106, 107, 109, 110, 111, 112, 113, 115, 116, 117, 118, 119, 120, 127, 128, 129, 130, 131, 132, 136, 137, 138, 139, 144, 145, 146, 147, 148, 159, 160, 161, 166, 167, 168, 178, 179, 180, 181, 182, 183, 184, 189, 190, 191, 192, 193, 198, 199, 200, 201, 205, 206, 209, 210, 216, 217, 218, 223, 224, 225, 226, 227, 232, 233, 234, 235, 236, 241, 242, 243], \
           []), \
           (u'Here are some', [4, 5, 8, 9], []), \
           (u'NBSPs!', [5], [])]
        expected_text_output = 'the term " anarchism " derives from the greek \xe1\xbc\x84\xce\xbd\xce\xb1\xcf\x81\xcf\x87\xce\xbf\xcf\x82 , " anarchos " , meaning " without rulers " , from the prefix \xe1\xbc\x80\xce\xbd - ( " an - " , " without " ) + \xe1\xbc\x80\xcf\x81\xcf\x87\xce\xae ( " arch\xc3\xaa " , " sovereignty , realm , magistracy " ) + - \xce\xb9\xcf\x83\xce\xbc\xcf\x8c\xcf\x82 ( " - ismos " , from the suffix - \xce\xb9\xce\xb6\xce\xb5\xce\xb9\xce\xbd , " - izein " " - izing " ) .\nhere are some\nnbsps !\n'
        self.run_assertions(text_to_segment_tokenise, expected_list_of_tuple_output, expected_text_output)

    def test_sentence_segmentation(self):

        text_to_segment_tokenise = u'Another libertarian tradition is that of unschooling and the free school in which child-led activity replaces pedagogic approaches. Experiments in Germany led to A. S. Neill founding what became Summerhill School in 1921. Summerhill is often cited as an example of anarchism in practice. However, although Summerhill and other free schools are radically libertarian, they differ in principle from those of Ferrer by not advocating an overtly-political class struggle-approach.\nThe Academy of Motion Picture Arts and Sciences itself was conceived by Metro-Goldwyn-Mayer studio boss Louis B. Mayer.  The 1st Academy Awards ceremony was held on Thursday, May 16, 1929, at the Hotel Roosevelt in Hollywood to honor outstanding film achievements of 1927 and 1928.\nWhen the Western Roman Empire collapsed, Berbers became independent again in many areas, while the Vandals took control over other parts, where they remained until expelled by the generals of the Byzantine Emperor, Justinian I. The Byzantine Empire then retained a precarious grip on the east of the country until the coming of the Arabs in the eighth century.'
        expected_list_of_tuple_output = [ \
            (u'Another libertarian tradition is that of unschooling and the free school in which child-led activity replaces pedagogic approaches.', \
            [7, 8, 19, 20, 29, 30, 32, 33, 37, 38, 40, 41, 52, 53, 56, 57, 60, 61, 65, 66, 72, 73, 75, 76, 81, 82, 87, 88, 91, 92, 100, 101, 109, 110, 119, 120, 130], \
            []), \
            (u'Experiments in Germany led to A. S. Neill founding what became Summerhill School in 1921.', \
            [11, 12, 14, 15, 22, 23, 26, 27, 29, 30, 32, 33, 35, 36, 41, 42, 50, 51, 55, 56, 62, 63, 73, 74, 80, 81, 83, 84, 88], \
            [(84, 4, u'<4-digit-integer>')]), \
            (u'Summerhill is often cited as an example of anarchism in practice.', \
            [10, 11, 13, 14, 19, 20, 25, 26, 28, 29, 31, 32, 39, 40, 42, 43, 52, 53, 55, 56, 64], \
            []), \
            (u'However, although Summerhill and other free schools are radically libertarian, they differ in principle from those of Ferrer by not advocating an overtly-political class struggle-approach.', \
            [7, 8, 9, 17, 18, 28, 29, 32, 33, 38, 39, 43, 44, 51, 52, 55, 56, 65, 66, 77, 78, 79, 83, 84, 90, 91, 93, 94, 103, 104, 108, 109, 114, 115, 117, 118, 124, 125, 127, 128, 131, 132, 142, 143, 145, 146, 153, 154, 163, 164, 169, 170, 178, 179, 187], \
            []), \
            (u'The Academy of Motion Picture Arts and Sciences itself was conceived by Metro-Goldwyn-Mayer studio boss Louis B. Mayer.', \
            [3, 4, 11, 12, 14, 15, 21, 22, 29, 30, 34, 35, 38, 39, 47, 48, 54, 55, 58, 59, 68, 69, 71, 72, 77, 78, 85, 86, 91, 92, 98, 99, 103, 104, 109, 110, 112, 113, 118], \
            []), \
            (u'The 1st Academy Awards ceremony was held on Thursday, May 16, 1929, at the Hotel Roosevelt in Hollywood to honor outstanding film achievements of 1927 and 1928.', \
            [3, 4, 7, 8, 15, 16, 22, 23, 31, 32, 35, 36, 40, 41, 43, 44, 52, 53, 54, 57, 58, 60, 61, 62, 66, 67, 68, 70, 71, 74, 75, 80, 81, 90, 91, 93, 94, 103, 104, 106, 107, 112, 113, 124, 125, 129, 130, 142, 143, 145, 146, 150, 151, 154, 155, 159], \
            [(4, 1, u'<1-digit-integer>'), (58, 2, u'<2-digit-integer>'), (62, 4, u'<4-digit-integer>'), (146, 4, u'<4-digit-integer>'), (155, 4, u'<4-digit-integer>')]), \
            (u'When the Western Roman Empire collapsed, Berbers became independent again in many areas, while the Vandals took control over other parts, where they remained until expelled by the generals of the Byzantine Emperor, Justinian I.', \
            [4, 5, 8, 9, 16, 17, 22, 23, 29, 30, 39, 40, 41, 48, 49, 55, 56, 67, 68, 73, 74, 76, 77, 81, 82, 87, 88, 89, 94, 95, 98, 99, 106, 107, 111, 112, 119, 120, 124, 125, 130, 131, 136, 137, 138, 143, 144, 148, 149, 157, 158, 163, 164, 172, 173, 175, 176, 179, 180, 188, 189, 191, 192, 195, 196, 205, 206, 213, 214, 215, 224, 225], \
            []), \
            (u'The Byzantine Empire then retained a precarious grip on the east of the country until the coming of the Arabs in the eighth century.', \
            [3, 4, 13, 14, 20, 21, 25, 26, 34, 35, 36, 37, 47, 48, 52, 53, 55, 56, 59, 60, 64, 65, 67, 68, 71, 72, 79, 80, 85, 86, 89, 90, 96, 97, 99, 100, 103, 104, 109, 110, 112, 113, 116, 117, 123, 124, 131], \
            [])]
        expected_text_output = 'another libertarian tradition is that of unschooling and the free school in which child - led activity replaces pedagogic approaches .\nexperiments in germany led to a. s. neill founding what became summerhill school in <4-digit-integer> .\nsummerhill is often cited as an example of anarchism in practice .\nhowever , although summerhill and other free schools are radically libertarian , they differ in principle from those of ferrer by not advocating an overtly - political class struggle - approach .\nthe academy of motion picture arts and sciences itself was conceived by metro - goldwyn - mayer studio boss louis b. mayer .\nthe <1-digit-integer>st academy awards ceremony was held on thursday , may <2-digit-integer> , <4-digit-integer> , at the hotel roosevelt in hollywood to honor outstanding film achievements of <4-digit-integer> and <4-digit-integer> .\nwhen the western roman empire collapsed , berbers became independent again in many areas , while the vandals took control over other parts , where they remained until expelled by the generals of the byzantine emperor , justinian i.\nthe byzantine empire then retained a precarious grip on the east of the country until the coming of the arabs in the eighth century .\n'
            
        self.run_assertions(text_to_segment_tokenise, expected_list_of_tuple_output, expected_text_output)


    def test_quotations_and_multiple_punctuation(self):

        text_to_segment_tokenise = u'Accordingly, "libertarian socialism" is sometimes used as a synonym for socialist anarchism, to distinguish it from "individualist libertarianism" (individualist anarchism). On the other hand, some use "libertarianism" to refer to individualistic free-market philosophy only, referring to free-market anarchism as "libertarian anarchism." '+"Citizens can oppose a decision ('besluit') made by a public body ('bestuursorgaan') within the administration\nThe Treaty could be considered unpopular in Scotland: Sir George Lockhart of Carnwath, the only member of the Scottish negotiating team against union, noted that `The whole nation appears against the Union' and even Sir John Clerk of Penicuik, an ardent pro-unionist and Union negotiator, observed that the treaty was `contrary to the inclinations of at least three-fourths of the Kingdom'."
        expected_list_of_tuple_output = [ \
            (u'Accordingly, "libertarian socialism" is sometimes used as a synonym for socialist anarchism, to distinguish it from "individualist libertarianism" (individualist anarchism).', \
            [11, 12, 13, 14, 25, 26, 35, 36, 37, 39, 40, 49, 50, 54, 55, 57, 58, 59, 60, 67, 68, 71, 72, 81, 82, 91, 92, 93, 95, 96, 107, 108, 110, 111, 115, 116, 117, 130, 131, 145, 146, 147, 148, 161, 162, 171, 172], \
            []), \
            (u'On the other hand, some use "libertarianism" to refer to individualistic free-market philosophy only, referring to free-market anarchism as "libertarian anarchism."', \
            [2, 3, 6, 7, 12, 13, 17, 18, 19, 23, 24, 27, 28, 29, 43, 44, 45, 47, 48, 53, 54, 56, 57, 72, 73, 77, 78, 84, 85, 95, 96, 100, 101, 102, 111, 112, 114, 115, 119, 120, 126, 127, 136, 137, 139, 140, 141, 152, 153, 162, 163], \
            []), \
            (u"Citizens can oppose a decision ('besluit') made by a public body ('bestuursorgaan') within the administration", \
            [8, 9, 12, 13, 19, 20, 21, 22, 30, 31, 32, 33, 40, 41, 42, 43, 47, 48, 50, 51, 52, 53, 59, 60, 64, 65, 66, 67, 81, 82, 83, 84, 90, 91, 94, 95], \
            []), \
            (u"The Treaty could be considered unpopular in Scotland: Sir George Lockhart of Carnwath, the only member of the Scottish negotiating team against union, noted that `The whole nation appears against the Union' and even Sir John Clerk of Penicuik, an ardent pro-unionist and Union negotiator, observed that the treaty was `contrary to the inclinations of at least three-fourths of the Kingdom'.", \
            [3, 4, 10, 11, 16, 17, 19, 20, 30, 31, 40, 41, 43, 44, 52, 53, 54, 57, 58, 64, 65, 73, 74, 76, 77, 85, 86, 87, 90, 91, 95, 96, 102, 103, 105, 106, 109, 110, 118, 119, 130, 131, 135, 136, 143, 144, 149, 150, 151, 156, 157, 161, 162, 163, 166, 167, 172, 173, 179, 180, 187, 188, 195, 196, 199, 200, 205, 206, 207, 210, 211, 215, 216, 219, 220, 224, 225, 230, 231, 233, 234, 242, 243, 244, 246, 247, 253, 254, 257, 258, 266, 267, 270, 271, 276, 277, 287, 288, 289, 297, 298, 302, 303, 306, 307, 313, 314, 317, 318, 319, 327, 328, 330, 331, 334, 335, 347, 348, 350, 351, 353, 354, 359, 360, 365, 366, 373, 374, 376, 377, 380, 381, 388, 389], \
            [])]
        expected_text_output = 'accordingly , " libertarian socialism " is sometimes used as a synonym for socialist anarchism , to distinguish it from " individualist libertarianism " ( individualist anarchism ) .\non the other hand , some use " libertarianism " to refer to individualistic free - market philosophy only , referring to free - market anarchism as " libertarian anarchism . "\n'+"citizens can oppose a decision ( ' besluit ' ) made by a public body ( ' bestuursorgaan ' ) within the administration\nthe treaty could be considered unpopular in scotland : sir george lockhart of carnwath , the only member of the scottish negotiating team against union , noted that ` the whole nation appears against the union ' and even sir john clerk of penicuik , an ardent pro - unionist and union negotiator , observed that the treaty was ` contrary to the inclinations of at least three - fourths of the kingdom ' .\n"

        self.run_assertions(text_to_segment_tokenise, expected_list_of_tuple_output, expected_text_output)

    def test_ints_with_or_without_following_punctuation(self):
        text_to_segment_tokenise = u'Hidehiko Shimizu (born 4 November 1954) is a former Japanese football player. He has played for Nissan Motors.'
        expected_list_of_tuple_output = [ \
            (u'Hidehiko Shimizu (born 4 November 1954) is a former Japanese football player.', \
            [8, 9, 16, 17, 18, 22, 23, 24, 25, 33, 34, 38, 39, 40, 42, 43, 44, 45, 51, 52, 60, 61, 69, 70, 76], \
            [(23, 1, u'<1-digit-integer>'), (34, 4, u'<4-digit-integer>')]), \
            (u'He has played for Nissan Motors.', \
            [2, 3, 6, 7, 13, 14, 17, 18, 24, 25, 31], \
            [])]
        expected_text_output = 'hidehiko shimizu ( born <1-digit-integer> november <4-digit-integer> ) is a former japanese football player .\nhe has played for nissan motors .\n'

        self.run_assertions(text_to_segment_tokenise, expected_list_of_tuple_output, expected_text_output)

    def test_end_of_document(self):
        text_to_segment_tokenise = u'---END.OF.DOCUMENT---'
        expected_list_of_tuple_output = [(u'---END.OF.DOCUMENT---', [1, 2, 3, 18, 19, 20], [])]
        expected_text_output = '- - - end.of.document - - -\n'
        self.run_assertions(text_to_segment_tokenise, expected_list_of_tuple_output, expected_text_output)

    def test_elipses(self):
        text_to_segment_tokenise = u"Elipses here... and there..."
        expected_list_of_tuple_output = [(u"Elipses here... and there...", [7, 8, 12, 15, 16, 19, 20, 25], [])]
        expected_text_output = 'elipses here ... and there ...\n'
        self.run_assertions(text_to_segment_tokenise, expected_list_of_tuple_output, expected_text_output)

    def test_single_quotes(self):
        text_to_segment_tokenise = u"this 'could' be"
        expected_list_of_tuple_output = [(u"this 'could' be", [4, 5, 6, 11, 12, 13], [])]
        expected_text_output = 'this \' could \' be\n'
        self.run_assertions(text_to_segment_tokenise, expected_list_of_tuple_output, expected_text_output)

    def test_mid_word(self):
        text_to_segment_tokenise = u'a line-or-three or 100,000.1 lines.  This&that.\nH. Amber Wilcox-O\'Hearn\n'+"They're his, not my brother's.\n3m/s"
        expected_list_of_tuple_output = [ \
            ( u'a line-or-three or 100,000.1 lines.', \
            [1, 2, 6, 7, 9, 10, 15, 16, 18, 19, 28, 29, 34], \
            [(19, 3, u'<3-digit-integer>'), (23, 3, u'<3-digit-integer>'), (27, 1, u'<1-digit-integer>')]), \
            (u'This&that.', [4, 5, 9], []), \
            (u'H. Amber Wilcox-O\'Hearn', [2, 3, 8, 9, 15, 16], []), \
            (u"They're his, not my brother's.", [7, 8, 11, 12, 13, 16, 17, 19, 20, 29], []), \
            (u"3m/s", [2, 3], [(0, 1, u'<1-digit-integer>')])]
        expected_text_output = 'a line - or - three or <3-digit-integer>,<3-digit-integer>.<1-digit-integer> lines .\nthis & that .\nh. amber wilcox - o\'hearn\nthey\'re his , not my brother\'s .\n<1-digit-integer>m / s\n'
        self.run_assertions(text_to_segment_tokenise, expected_list_of_tuple_output, expected_text_output)

    def test_multiple_substitutions_at_the_end(self):
        text_to_segment_tokenise = u'What about 100,000'
        expected_list_of_tuple_output = [(u'What about 100,000', [4, 5, 10, 11], [(11, 3, u'<3-digit-integer>'), (15, 3, u'<3-digit-integer>')])]
        expected_text_output = 'what about <3-digit-integer>,<3-digit-integer>\n'
        self.run_assertions(text_to_segment_tokenise, expected_list_of_tuple_output, expected_text_output)

    def test_sentence_final_punctuation(self):
        text_to_segment_tokenise = u'Finally.\nFinally?\nFinally!'
        expected_list_of_tuple_output = [ \
            (u'Finally.', [7], []),
            (u'Finally?', [7], []),
            (u'Finally!', [7], [])]
        expected_text_output = 'finally .\nfinally ?\nfinally !\n'
        self.run_assertions(text_to_segment_tokenise, expected_list_of_tuple_output, expected_text_output)

    def test_dollar_percent_and_strings_of_consecutive_numbers(self):
        text_to_segment_tokenise = u"$1.50, 30%"
        expected_list_of_tuple_output = [(u"$1.50, 30%", [1, 5, 6, 7, 9], \
             [(1, 1, u'<1-digit-integer>'), (3, 2, u'<2-digit-integer>'), (7, 2, u'<2-digit-integer>')])]
        expected_text_output = '$ <1-digit-integer>.<2-digit-integer> , <2-digit-integer> %\n'
        self.run_assertions(text_to_segment_tokenise, expected_list_of_tuple_output, expected_text_output)

    def test_line_starts_with_punct_or_number(self):
        text_to_segment_tokenise = u'"This 34.\n'
        expected_list_of_tuple_output = [(u'"This 34.', [1, 5, 6, 8], [(6, 2, u'<2-digit-integer>')])]
        expected_text_output = '" this <2-digit-integer> .\n'
        self.run_assertions(text_to_segment_tokenise, expected_list_of_tuple_output, expected_text_output)

    def test_abbreviations(self):
        # This is, of course, incorrectly segmented, but correctly characterises the behaviour of the segmenter.
        text_to_segment_tokenise = u'Mr. Shimizu was not born in the U.S. "You are just joking."'
        expected_list_of_tuple_output = [ \
            (u'Mr. Shimizu was not born in the U.S. "You are just joking."', \
            [3, 4, 11, 12, 15, 16, 19, 20, 24, 25, 27, 28, 31, 32, 36, 37, 38, 41, 42, 45, 46, 50, 51, 57, 58], \
            [])]
        expected_text_output = 'mr. shimizu was not born in the u.s. " you are just joking . "\n'
        self.run_assertions(text_to_segment_tokenise, expected_list_of_tuple_output, expected_text_output)

    def test_initials(self):
        text_to_segment_tokenise = u'Neither was J. S. Bach.'
        expected_list_of_tuple_output = [(u'Neither was J. S. Bach.', [7, 8, 11, 12, 14, 15, 17, 18, 22], [])]
        expected_text_output = 'neither was j. s. bach .\n'
        self.run_assertions(text_to_segment_tokenise, expected_list_of_tuple_output, expected_text_output)

    def test_multiple_spaces_space_at_beginning_of_line(self):
        text_to_segment_tokenise = u'Extra  spaces     here \n and here'
        expected_list_of_tuple_output = [ \
            (u'Extra spaces here', [5, 6, 12, 13], []), \
            (u'and here', [3, 4], [])]
        expected_text_output = 'extra spaces here\nand here\n'
        self.run_assertions(text_to_segment_tokenise, expected_list_of_tuple_output, expected_text_output)
        
    def test_suffixes(self):
        text_to_segment_tokenise = u'Don\'t keep this together: -suffix'
        expected_list_of_tuple_output = [(u'Don\'t keep this together: -suffix', [5, 6, 10, 11, 15, 16, 24, 25, 26, 27], [])]
        expected_text_output = 'don\'t keep this together : - suffix\n'
        self.run_assertions(text_to_segment_tokenise, expected_list_of_tuple_output, expected_text_output)

    def test_commandline(self):
        segment_and_tokenise = subprocess.Popen(['python', 'recluse/nltk_based_segmenter_tokeniser.py'], stdin=-1, stdout=-1, stderr=-1)
        (stdoutdata, stderrdata) = segment_and_tokenise.communicate(input="this's a test\" and. so is 1984.")
        self.assertEqual(segment_and_tokenise.returncode, 0)
        self.assertEqual(stdoutdata, 'this\'s a test " and .\nso is <4-digit-integer> .\n'), stdoutdata


if __name__ == '__main__':
    unittest.main()



# test_utils.py

import unittest, os, shutil
from recluse import utils


class UtilsTest(unittest.TestCase):

    def test_open_with_unicode(self):

        infile_obj = utils.open_with_unicode('recluse/test/data/segmenter_training', None, 'r')
        first_line = infile_obj.readline()
        self.assertEqual(first_line, u'\n'), first_line
        self.assertEqual(type(first_line), unicode), type(first_line)

        infile_obj = utils.open_with_unicode('recluse/test/data/small_westbury.txt.bz2', 'bzip2', 'r')
        first_line = infile_obj.readline()
        self.assertEqual(first_line, u'Anarchism.\n'), first_line
        self.assertEqual(type(first_line), unicode), type(first_line)

        infile_obj = utils.open_with_unicode('recluse/test/data/unigram_counts.gz', 'gzip', 'r')
        first_line = infile_obj.readline()
        self.assertEqual(first_line, u'funeral 4\n'), first_line
        self.assertEqual(type(first_line), unicode), type(first_line)

    def test_split_file_into_chunks(self):

        infile_name = 'recluse/test/data/unigram_counts.bz2'
        temp_dir_name = 'TEMP_DIR'
        while os.path.isdir(temp_dir_name):
            temp_dir_name += 'x'
        temp_dir_name += '/'
        os.mkdir(temp_dir_name)
        utils.split_file_into_chunks(infile_name, temp_dir_name, 5)
        self.assertTrue(os.path.isfile(temp_dir_name + '/000.bz2'))
        self.assertTrue(os.path.isfile(temp_dir_name + '/001.bz2'))
        self.assertTrue(os.path.isfile(temp_dir_name + '/002.bz2'))
        self.assertTrue(os.path.isfile(temp_dir_name + '/003.bz2'))
        self.assertTrue(os.path.isfile(temp_dir_name + '/004.bz2'))
        infile_obj = utils.open_with_unicode(temp_dir_name + '/003.bz2', 'bzip2', 'r')
        first_line  = infile_obj.readline()
        self.assertEqual(first_line, u'holy 2\n'), first_line
        shutil.rmtree(temp_dir_name)
        

if __name__ == '__main__':
    unittest.main()



# vocabulary_generator.py

import subprocess, os, shutil, sys
from utils import *
import vocabulary_cutter

class VocabularyGenerator():

    """
    Uses the SRILM toolkit to generate unigram counts and then cuts
    down to size with the most frequent words.
    """

    def __init__(self, lines_per_chunk=100000):
        self.lines_per_chunk = lines_per_chunk
        self.temporary_data_directory = 'TEMP_DATA_DIR/'

    def srilm_make_merge_batch_counts(self, chunk_directory):

        self.temporary_counts_directory = self.temporary_data_directory + 'TEMP_COUNTS_DIR/'
        os.mkdir(self.temporary_counts_directory)
        file_names_file_obj = open(chunk_directory + 'file_names', 'w')
        file_names_file_obj.writelines([chunk_directory + s + '\n' for s in os.listdir(chunk_directory) if s != 'file_names'])
        file_names_file_obj.close()
        srilm_make_batch_counts = subprocess.call(['make-batch-counts', chunk_directory + 'file_names', '1', 'recluse/nltkbasedsegmentandtokenise.sh', self.temporary_counts_directory, '-write-order 1'])
        srilm_merge_batch_counts = subprocess.call(['merge-batch-counts', self.temporary_counts_directory])

    def generate_vocabulary(self, text_file_name, size, vocabulary_file_obj):
        
        assert not os.path.isdir(self.temporary_data_directory)
        os.mkdir(self.temporary_data_directory)

        temporary_chunk_directory = self.temporary_data_directory + 'TEMP_CHUNK_DIR/'
        os.mkdir(temporary_chunk_directory)
        split_file_into_chunks(text_file_name, temporary_chunk_directory, self.lines_per_chunk) 

        self.srilm_make_merge_batch_counts(temporary_chunk_directory)
        merged_counts_file = [f for f in os.listdir(self.temporary_counts_directory) if f.endswith('.ngrams.gz')][0]
        unigram_counts_file_obj = open_with_unicode(self.temporary_counts_directory + merged_counts_file, 'gzip', 'r')
        cutter = vocabulary_cutter.VocabularyCutter(unigram_counts_file_obj, vocabulary_file_obj)
        cutter.cut_vocabulary(size)
        
        shutil.rmtree(self.temporary_data_directory)


if __name__ == '__main__':
    infile_name = sys.argv[1]
    size = int(sys.argv[2])
    outfile_obj = sys.stdout
    vg = VocabularyGenerator()
    vg.generate_vocabulary(infile_name, size, outfile_obj)



#!/usr/bin/env python

from distutils.core import setup
import versioneer

versioneer.versionfile_build='recluse/_version.py',
versioneer.versionfile_source='code/recluse/_version.py'
versioneer.tag_prefix = 'recluse-'
versioneer.parentdir_prefix = 'recluse-'


setup(name='recluse',
      description='Reproducible Experimentation for Computational Linguistics Use',
      long_description=open('README.rst').read(),
      author='L. Amber Wilcox-O\'Hearn',
      author_email='amber@cs.toronto.edu',
      url='https://github.com/lamber/recluse',
      package_dir = {'':'code'},
      packages=['recluse', 'recluse.test'],
      license='COPYING',
      version=versioneer.get_version(),
      cmdclass=versioneer.get_cmdclass()
     )
